var getUserDetails = null

!function () {
    var origin = "https://begenieus.com"
    var tutorShotId = null
    var imageNumber = 0;
    var recording = false;
    var loggedIn = false;
    var userDetails = {}


    function setData(key, value, callback) {
        return new Promise((res, rej) => {
            try {
                chrome.storage.local.set({ [key]: value }, () => {
                    if (callback) callback(value)
                    else res(value)
                })
            } catch (err) {
                if (callback) callback(undefined, err)
                else callback(undefined)
            }
        })
    }

    function getData(key, callback) {
        return new Promise((res, rej) => {
            try {
                chrome.storage.local.get(key, (data) => {
                    if (callback) callback(data[key])
                    else res(data[key])
                })
            } catch (err) {
                if (callback) callback(undefined, err)
                else res(undefined)
            }
        })
    }

    getData('userDetails', async (data) => {
        if (data) {
            let tso = await getData('tutorShotId')
            tutorShotId = tso ? tso : null
            let imgNum = await getData('imageNumber')
            imageNumber = imgNum ? imgNum : 0
            let rec = await getData('recording')
            recording = rec ? rec : false
            userDetails = data
            loggedIn = (userDetails.email || userDetails.uid) ? true : false;

        }
    })

    function getStatus() {
        return tutorShotId ? (recording ? "recording" : "paused") : "stopped"
    }


    function createAPI(origin) {
        var baseURL = origin
        function objectToQueryString(obj) {
            const keyValuePairs = [];

            for (const key in obj) {
                if (obj.hasOwnProperty(key)) {
                    const value = obj[key];
                    if (value !== undefined) {
                        keyValuePairs.push(encodeURIComponent(key) + '=' + encodeURIComponent(value));
                    }
                }
            }

            return keyValuePairs.join('&');
        }
        return {
            post(url, body = {}, headers = {}) {
                if (url.slice(0, 4) != 'http') url = (baseURL + `/api${url[0] == '/' ? '' : '/'}${url}`)
                return new Promise((res, rej) => {
                    let bodyObj = { ...body }
                    let headerObj = {
                        'Content-Type': 'application/json',
                        'userid': userDetails?.uid,
                        ...headers,
                    }
                    fetch(url, {
                        method: 'POST',
                        body: JSON.stringify({ ...bodyObj }),
                        headers: headerObj,
                    }).then(async (data) => {
                        if (headerObj['Content-Type'] == 'application/json') {
                            res(await data.json())
                        } else {
                            res(data)
                        }
                    }).catch(err => {
                        console.log('fetch err', err)
                        rej(err)
                    })
                })
            },
            put(url, body = {}, headers = {}) {
                if (url.slice(0, 4) != 'http') url = baseURL + `/api${url[0] == '/' ? '' : '/'}${url}`
                return new Promise((res, rej) => {
                    let bodyObj = { ...body }
                    let headerObj = {
                        'Content-Type': 'application/json',
                        'userid': userDetails?.uid,
                        ...headers,
                    }
                    fetch(url, {
                        method: 'PUT',
                        body: JSON.stringify({ ...bodyObj }),
                        headers: headerObj,
                    }).then(async (data) => {
                        if (headerObj['Content-Type'] == 'application/json') {
                            res(await data.json())
                        } else {
                            res(data)
                        }
                    }).catch(err => {
                        console.log('fetch err', err)
                        rej(err)
                    })
                })
            },
            delete(url, obj = { params: {}, headers: {} }) {
                if (url.slice(0, 4) != 'http') url = baseURL + `/api${url[0] == '/' ? '' : '/'}${url}`
                let params = objectToQueryString(obj.params)
                if (params) url += `?${params}`
                return new Promise((res, rej) => {
                    let headerObj = {
                        'Content-Type': 'application/json',
                        'userid': userDetails?.uid,
                        ...obj.headers,
                    }
                    fetch(url, {
                        method: 'DELETE',
                        headers: headerObj,
                    }).then(async (data) => {
                        if (headerObj['Content-Type'] == 'application/json') {
                            res(await data.json())
                        } else {
                            res(data)
                        }
                    }).catch(err => {
                        console.log('fetch err', err)
                        rej(err)
                    })
                })
            },
            get(url, obj = { params: {}, headers: {} }) {
                if (url.slice(0, 4) != 'http') url = baseURL + `/api${url[0] == '/' ? '' : '/'}${url}`
                let params = objectToQueryString(obj.params)
                if (params) url += `?${params}`
                return new Promise((res, rej) => {
                    let headerObj = {
                        'Content-Type': 'application/json',
                        'userid': userDetails?.uid,
                        ...obj.headers,
                    }
                    fetch(url, {
                        method: 'DELETE',
                        headers: headerObj,
                    }).then(async (data) => {
                        if (headerObj['Content-Type'] == 'application/json') {
                            res(await data.json())
                        } else {
                            res(data)
                        }
                    }).catch(err => {
                        console.log('fetch err', err)
                        rej(err)
                    })
                })
            },
            objectToQueryString
        }
    }

    var api = createAPI(origin)


    async function onMessage(request, sender, sendResponse) {

        if (request.action === 'login') {
            userDetails = request.userDetails
            // check if credentials changed.
            if (request.userDetails.email != userDetails.email) {
                imageNumber = 0
            }
            loggedIn = true
            sendResponse({
                message: "Logged In",
                success: true,
                userDetails,
                tutorShotOid: tutorShotId,
                imageNumber: imageNumber,
                status: getStatus(),

            })
            sendMessageToActiveTab("loggedIn")
            setData("userDetails", userDetails)
            setData("recording", false)
            setData("tutorShotId", tutorShotId)
            setData("imageNumber", imageNumber)
            return
        }

        if (request.action === 'checkStatus') {
            sendResponse({
                userDetails,
                loggedIn,
                recording,
                tutorShotOid: tutorShotId,
                imageNumber: imageNumber,
                status: getStatus()
            })
            return
        }

        if (loggedIn && request.action === 'startRecording') {
            imageNumber = 0
            recording = true;
            tutorShotId = request.tutorShotId
            sendResponse({
                userDetails,
                loggedIn,
                recording,
                status: getStatus()
            });
            sendMessageToActiveTab("recordingStarted")
            setData("recording", true)
            setData("tutorShotId", tutorShotId)
            setData("imageNumber", imageNumber)
            return
        }
        if (loggedIn && request.action === 'pauseRecording') {
            recording = false;
            sendResponse({
                userDetails,
                loggedIn,
                recording,
                status: getStatus()
            });
            sendMessageToActiveTab("recordingPaused")
            setData("recording", false)
            return
        }
        if (loggedIn && request.action === 'resumeRecording') {
            recording = true;
            sendResponse({
                userDetails,
                loggedIn,
                recording,
                status: getStatus()
            });
            sendMessageToActiveTab("recordingResumed")
            setData("recording", true)
            return
        }
        if (loggedIn && request.action === 'stopRecording') {
            recording = false;
            let generatingOid = tutorShotId
            if (imageNumber < 1) {
                let { data } = await api.delete("tutor-shot", {
                    params: { _id: tutorShotId },
                    headers: { userid: userDetails.uid }
                })
                tutorShotId = null
                sendMessageToActiveTab("showRestartProcess")
                setData("recording", false)
                setData("tutorShotId", null)
                return
            }
            else {
                tutorShotId = null
                sendResponse({
                    userDetails,
                    loggedIn,
                    recording,
                    status: "generatingVideo"
                });
                sendMessageToActiveTab("generatingVideo")
                await api.post('/tutor-shot/finalize', { _id: generatingOid }, { headers: { userid: userDetails.uid } })
                setData("recording", false)
                setData("imageNumber", 0)
                setData("tutorShotId", null)
                imageNumber = 0;
                return
            }
        }
        if (loggedIn && request.action === 'limitExceed') {
        sendMessageToActiveTab("limitExceed")
        return
        }
        if (request.action === 'logout') {
            loggedIn = false;
            recording = false;
            userDetails = {};
            tutorShotId = null;
            imageNumber = 0;
            sendResponse({
                userDetails,
                loggedIn,
                recording,
                status: "stopped"
            });
            sendMessageToActiveTab("loggedOut")
            setData("recording", false)
            setData("imageNumber", 0)
            setData("tutorShotId", null)
            setData("userDetails", {})
            setData("loggedIn", false)
            return
        }
        if (loggedIn && request.action === 'restartRecording') {
            imageNumber = 0;
            sendResponse({
                userDetails,
                loggedIn,
                recording,
                status: getStatus()
            });
            await api.delete('/tutor-shot/segments', { params: { tutorShotOid: tutorShotId } })
            sendMessageToActiveTab("recordingRestarted")
            setData("imageNumber", 0)
            return
        }

        if (request.action === 'captureSS') {
            if (!loggedIn) {
                return;
            }
            if (!recording) {
                return;
            }
            if (!tutorShotId) {
                return
            }
            chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
                if (tabs && tabs.length) { // if any tab is active
                    console.log("🚀 ~ tabs:", tabs)
                    var activeTab = tabs[0]
                    chrome.tabs.captureVisibleTab({ format: "png" }, async function (dataUrl) {
                        let message = {
                            ...userDetails,
                            tutorShotOid: tutorShotId,
                            action: 'SSCaptured',
                            dataUrl: dataUrl, imageNumber: imageNumber++,
                            status: getStatus(),
                            ...request.captureInfo ? request.captureInfo : {}
                        }
                        setData("imageNumber", imageNumber)
                        // to send message to the content instance of the
                        chrome.tabs.sendMessage(activeTab.id, message);
                        let data = await api.post('/tutor-shot/screen-shot', message)
                    })
                }
            })
            return
        }


    }
    chrome.runtime.onMessage.addListener(onMessage)

    getUserDetails = function () {
        let dataSet = {
            userDetails: JSON.parse(JSON.stringify(userDetails)),
            recording: JSON.parse(JSON.stringify(recording)),
            loggedIn: JSON.parse(JSON.stringify(loggedIn)),
            tutorShotId: JSON.parse(JSON.stringify(tutorShotId)),
            imageNumber: JSON.parse(JSON.stringify(imageNumber)),
        }
        return dataSet
    }

    function sendMessageToActiveTab(messageObj) {
        try {
            chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
                let activeTab = tabs[0]
                if (activeTab?.url?.slice(0, 6) != 'chrome') { // if any tab is active
                    let message = {
                        userDetails,
                        tutorShotOid: tutorShotId,
                        imageNumber: imageNumber,
                        status: getStatus(),
                        message: messageObj
                    }
                    try {
                        chrome.tabs.sendMessage(activeTab?.id, message);
                    } catch (err) {
                        console.log('error while sending data', activeTab, message)
                        console.log(err)
                    }
                }
            })
        } catch (err) {
            console.log(err)
        }
    }
}()
