!function () {
    const origin = "https://begenieus.com"

    const api = axios.create({
        baseURL: `${origin}/api`, // the base URL for nodeserver is okay but NGINX is giving error
        headers: {
            'Access-Control-Allow-Origin': '*', // Replace '*' with the origin you need
            'Content-Type': 'application/json',
        },
    });


    var loginCount = 0;
    let recording = false;
    let loggedIn = false;
    let startBtn = document.getElementById('start-btn')
    let stopBtn = document.getElementById('stop-btn');
    let userDetailsDiv = document.getElementById('user-details')
    let userName = document.getElementById('user-name')
    let userEmail = document.getElementById('user-email')
    let myCapturesBtn = document.getElementById('my-captures-btn')
    let logoutBtn = document.getElementById('logout-btn')
    myCapturesBtn.addEventListener('click',(e)=>{ window.open(origin+'/tutor-shot-list') })
    let userDetails = {};
    let maxTutorShotLimit = {};
    let savedTutorShot = {};

    let inProgress = false;

    logoutBtn.addEventListener('click', ()=>{
        chrome.runtime.sendMessage({ action: "logout" }, async function (response) {
            loggedIn = false
            userDetails.email = ""
            userDetails.uid = ""
            modifyPopupUI()
        })
    })

    let loginBtn = document.getElementById('login-btn')
    loginBtn.addEventListener('click', () => { window.open(origin) })

    startBtn.addEventListener('click', async function () {
        if (inProgress){
            return
        }
        inProgress = true
        try{
            let limitExceed = await getSubscriptions()
            if(limitExceed){
                window.close()
                chrome.runtime.sendMessage({ action: "limitExceed" }, async function (response) {
                    loggedIn = false
                    userDetails.email = ""
                    userDetails.uid = ""
                    modifyPopupUI()
                })
                return;
            }
            let { data } = await api.post('/tutor-shot', { title: "No Title" }, { headers: { userId: userDetails.uid } } )
            if (data.success){
                chrome.runtime.sendMessage({ action: "startRecording", tutorShotId:data.tutorShot._id }, async function (response) {
                    recording = response.recording
                    modifyPopupUI()
                    inProgress = false
                    window.close()
                })
            }
        }catch(err){
            console.log('error while starting to record', err)
            inProgress = false
        }

    })
    stopBtn.addEventListener('click', function () {

        if (inProgress){
            return
        }
        inProgress = true
        chrome.runtime.sendMessage({ action: "stopRecording" }, function (response) {
            recording = response.recording
            modifyPopupUI()
            inProgress = false
        })
    })

    function modifyPopupUI() {
        loginBtn.style.display = loggedIn ? 'none' : ''
        startBtn.style.display = loggedIn ? (recording ? 'none' : '') : 'none'
        stopBtn.style.display = loggedIn ? (!recording ? 'none' : '') : 'none'
        if (userDetails.email && userDetails.uid){
            userDetailsDiv.style.display = 'block'
            userEmail.innerHTML = userDetails.email
            userName.innerHTML = userDetails.name
        }else{
            userDetailsDiv.style.display = 'none'
            userEmail.innerHTML = ""
            userName.innerHTML = ""
        }

    }
    let getAuthHeader = function(){
        return {
            headers: {
                username: userDetails.name,
                userid: userDetails.uid,
                useroid: userDetails.userOid,
            }
        }
    }
        async function getSubscriptions(){
            try{
                if(!userDetails.uid) return  
                let { data } = await api.get("common/subscriptions", { headers: getAuthHeader().headers })
                if (data.success){
                const moduleObject = data.subscriptionData.feature.find(obj => obj.module === 'TUTOR_SHOT');
                maxTutorShotLimit = moduleObject ? moduleObject.quantity : null;
                api.defaults.headers.userId = userDetails.uid
                const tutorData = await api.get("tutor-shot/")
                savedTutorShot = tutorData.data.list.length
                if(maxTutorShotLimit<=savedTutorShot) return true
                else return false
                }
            }catch(err){
                console.log(err)
            }
        }

    function domLoadHandler() {
        chrome.runtime.sendMessage({ action: "checkStatus" }, function (response) {
            userDetails = response.userDetails;
            loggedIn = response.loggedIn, recording = response.recording
            modifyPopupUI()
            if(!Object.keys(userDetails).length && loginCount++<2){
                let extLoader = document.getElementById('ext-loader')
                extLoader.style.display = 'flex'
                let loginContainer = document.getElementById('login-btn')?.parentNode
                if (loginContainer) loginContainer.style.display = 'none'
                setTimeout( ()=>{
                    if (loginContainer) loginContainer.style.display = 'flex'
                    extLoader.style.display = 'none'
                    domLoadHandler()
                }, 500 )
            }
        })
    }
    // this code should run everytime the popup is opened
    document.addEventListener("DOMContentLoaded", domLoadHandler);



}()
